@@ -6,12 +6,21 @@ import pathlib
 import uuid
 import time
 import shutil
+import platform
 from dataclasses import dataclass, field, asdict
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Optional, Callable
 
 from rich.console import Console
 from rich.panel import Panel
 from rich.markdown import Markdown
+try:
+    from rich import __version__ as _rich_version  # noqa: F401
+except Exception:  # pragma: no cover - tests may stub out rich
+    import rich as _rich
+    if not hasattr(_rich, "__version__"):
+        _rich.__version__ = "0"
+    if not hasattr(_rich, "__file__"):
+        _rich.__file__ = "rich-not-installed"
 try:
     from rich import box  # type: ignore
 except Exception:  # pragma: no cover - tests stub out rich
@@ -27,6 +36,18 @@ from . import tools, models, openai_compat
 from .models import Model, OpenAIModel
 from .persona import Persona
 
+# optional custom builder for the system message
+_SYSTEM_MSG_BUILDER: Optional[Callable[[Persona, Optional[List[str]]], str]] = None
+
+
+def set_system_message_builder(
+    builder: Optional[Callable[[Persona, Optional[List[str]]], str]]
+) -> None:
+    """Register a callable to build the system prompt."""
+
+    global _SYSTEM_MSG_BUILDER
+    _SYSTEM_MSG_BUILDER = builder
+
 DEFAULT_PERSONA = Persona(
     os.getenv("PYGENT_PERSONA_NAME", "Pygent"),
     os.getenv("PYGENT_PERSONA", "a sandboxed coding assistant."),
@@ -36,6 +57,9 @@ DEFAULT_PERSONA = Persona(
 def build_system_msg(persona: Persona, disabled_tools: Optional[List[str]] = None) -> str:
     """Build the system prompt for ``persona`` with the active tools."""
 
+    if _SYSTEM_MSG_BUILDER:
+        return _SYSTEM_MSG_BUILDER(persona, disabled_tools)
+
     # Active tool schemas
     schemas = [
         s for s in tools.TOOL_SCHEMAS
@@ -50,6 +74,7 @@ def build_system_msg(persona: Persona, disabled_tools: Optional[List[str]] = Non
         user = os.getenv("USER", "unknown")
     dynamic_lines = [
         f"User: {user}",
+        f"OS: {platform.system()}",
         f"Working directory: {os.getcwd()}",
     ]
     if shutil.which("rg"):
@@ -227,23 +252,28 @@ class Agent:
                     console.print(
                         Panel(
                             f"$ {cmd}",
-                            title=f"{self.persona.name} pending bash",
-                            box=box.ROUNDED if box else None,
+                            title=f"[bold yellow]{self.persona.name} pending bash[/]",
+                            border_style="yellow",
+                            box=box.HEAVY_HEAD if box else None,
+                            title_align="left",
                         )
                     )
                     prompt = "Run this command?"
                     if questionary:
-                        ok = questionary.confirm(prompt).ask()
+                        ok = questionary.confirm(prompt, default=True).ask()
                     else:  # pragma: no cover - fallback for tests
-                        ok = input(f"{prompt} [y/N]: ").lower().startswith("y")
+                        ok_input = console.input(f"{prompt} [Y/n]: ").lower()
+                        ok = ok_input == "" or ok_input.startswith("y")
                     if not ok:
-                        output = f"$ {cmd}\n[aborted]"
+                        output = f"$ {cmd}\n[bold red]Aborted by user.[/]"
                         self.append_history({"role": "tool", "content": output, "tool_call_id": call.id})
                         console.print(
                             Panel(
                                 output,
-                                title=f"{self.persona.name} tool:{call.function.name}",
+                                title=f"[bold red]{self.persona.name} tool:{call.function.name}[/]",
+                                border_style="red",
                                 box=box.ROUNDED if box else None,
+                                title_align="left",
                             )
                         )
                         continue
@@ -260,21 +290,31 @@ class Agent:
                     {"role": "tool", "content": output, "tool_call_id": call.id}
                 )
                 if call.function.name not in {"ask_user", "stop"}:
+                    display_output = output
+                    if call.function.name == "read_image" and output.startswith("data:image"):
+                        try:
+                            args = json.loads(call.function.arguments or "{}")
+                            path = args.get("path", "<unknown>")
+                        except Exception:
+                            path = "<unknown>"
+                        display_output = f"returned data URL for {path}"
                     console.print(
                         Panel(
-                            output,
-                            title=f"{self.persona.name} tool:{call.function.name}",
+                            display_output,
+                            title=f"[bold bright_blue]{self.persona.name} tool:{call.function.name}[/]",
+                            border_style="bright_blue",
                             box=box.ROUNDED if box else None,
+                            title_align="left",
                         )
                     )
         else:
-            markdown_response = Markdown(assistant_msg.content)
+            markdown_response = Markdown(assistant_msg.content or "") # Ensure content is not None
             console.print(
                 Panel(
                     markdown_response,
-                    title=f"{self.persona.name} replied",
+                    title=f"[bold green]{self.persona.name} replied[/]",
                     title_align="left",
-                    border_style="cyan",
+                    border_style="green",
                     box=box.ROUNDED if box else None,
                 )
             )
@@ -352,15 +392,16 @@ def run_interactive(
     from .commands import COMMANDS
     mode = "Docker" if agent.runtime.use_docker else "local"
     console.print(
-        f"[bold green]{agent.persona.name} ({mode})[/] started. (type /exit to quit)"
+        f"[bold green]{agent.persona.name} ({mode})[/] started. (Type /exit to quit)"
     )
-    console.print("Type /help for available commands.")
+    console.print("Type /help for a list of available commands.")
     try:
         next_msg: Optional[str] = None
         while True:
             if next_msg is None:
-                user_msg = console.input("[cyan]user> [/]")
+                user_msg = console.input("[bold steel_blue]>>> [/]")
             else:
+                console.print(f"[bold steel_blue]>>> [/]{next_msg}", highlight=False)
                 user_msg = next_msg
                 next_msg = None
             if agent._log_fp:
@@ -369,8 +410,11 @@ def run_interactive(
                     agent._log_fp.flush()
                 except Exception:
                     pass
-            cmd = user_msg.split(maxsplit=1)[0]
-            args = user_msg[len(cmd):].strip() if " " in user_msg else ""
+            if not user_msg.strip():
+                continue
+            parts = user_msg.split(maxsplit=1)
+            cmd = parts[0]
+            args = parts[1] if len(parts) == 2 else ""
             if cmd in {"/exit", "quit", "q"}:
                 break
             if cmd in COMMANDS:
@@ -397,9 +441,17 @@ def run_interactive(
         dest = pathlib.Path.cwd() / f"crash_{uuid.uuid4().hex[:8]}"
         cmd_save(agent, str(dest))
         console.print(
-            f"[red]Error: {exc}. Workspace saved to {dest}. Load with `pygent --load {dest}`[/]"
+            Panel(
+                f"An unexpected error occurred: [bold red]{exc}[/]\n"
+                f"Your workspace has been saved to [cyan]{dest}[/].\n"
+                f"You can restore it using: [bold yellow]pygent --load {dest}[/]",
+                title="[bold red]Critical Error[/]",
+                border_style="red",
+                box=box.DOUBLE if box else None,
+            )
         )
-        raise
+        # raise # Optionally re-raise the exception if needed for debugging or higher-level handling
     finally:
+        console.print("[dim]Closing session...[/]")
         agent.close()
         agent.runtime.cleanup()
